/*
SDX: Documentary System in XML.
Copyright (C) 2000, 2001, 2002  Ministere de la culture et de la communication (France), AJLSM

Ministere de la culture et de la communication,
Mission de la recherche et de la technologie
3 rue de Valois, 75042 Paris Cedex 01 (France)
mrt@culture.fr, michel.bottin@culture.fr

AJLSM, 17, rue Vital Carles, 33000 Bordeaux (France)
sevigny@ajlsm.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
or connect to:
http://www.fsf.org/copyleft/gpl.html
*/
package fr.gouv.culture.sdx.search.lucene.query;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.search.*;
import org.apache.lucene.search.Query;

import java.io.IOException;
import java.util.ArrayList;

import fr.gouv.culture.sdx.utils.Utilities;

/*Rasik Pandey rasik.pandey@ajlsm.com*/

/**Simple wrapper for a Lucene query that
 * collects all queries generated by calling
 * rewrite on the original Lucene query and stores
 * them in a BooleanQuery.
 *
 * A Searcher will call the rewrite() method
 * for each index and hence generate a query
 * containing terms for the respective index. This
 * class collects these queries so that they may be
 * used for highlighting, query expansion, etc. by
 * retrieving the underlying terms.
 *
 * @see #rewrite
 * @see #getRewrittenQueries
 * @see #getOriginalQuery
 */
public class CachingRewrittenQueryWrapper extends Query {
    protected org.apache.lucene.search.Query originalQuery = null;
    protected ArrayList rewrittenQueries = new ArrayList();

    public CachingRewrittenQueryWrapper(Query originalQuery) {
        this.originalQuery = originalQuery;
    }

    public BooleanQuery getRewrittenQueries() {
        BooleanQuery l_rewrittenQueries = Utilities.newBooleanQuery();
        for (int i = 0; i < rewrittenQueries.size(); i++) {
            Query rewrittenQuery = (Query) rewrittenQueries.get(i);
            l_rewrittenQueries.add(rewrittenQuery, false, false);
        }
        return  l_rewrittenQueries;
    }

    public void resetRewrittenQueries() {
        this.rewrittenQueries = new ArrayList();
    }

    public Query getOriginalQuery() {
        return this.originalQuery;
    }

    public Query rewrite(IndexReader reader) throws IOException {
        Query rewrittenQuery = this.originalQuery.rewrite(reader);
        if (!this.rewrittenQueries.contains(rewrittenQuery))
            this.rewrittenQueries.add(rewrittenQuery);
        return rewrittenQuery;
    }

    public Query combine(Query[] queries) {
        return this.originalQuery.combine(queries);
    }

    public Similarity getSimilarity(Searcher searcher) {
        return this.originalQuery.getSimilarity(searcher);
    }

    protected void finalize() throws Throwable {
        super.finalize();
        //TODO maybe something here to ensure that all resources held by rewrittenQueries are cleaned up properly
    }

    public String toString() {
        return this.originalQuery.toString();
    }

    public String toString(String field) {
        return this.originalQuery.toString(field);
    }
}
